/*
 * ICHybridRecommender.java
 *
 * Created on 6 de Fevereiro de 2007, 22:09
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.unicamp.ic.recommender.hybrid;

import br.unicamp.ic.recommender.engine.BasicEngine;

import br.unicamp.ic.recommender.entity.User;
import br.unicamp.ic.recommender.entity.Item;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Daniel
 */
public class ICHybridRecommender extends BasicEngine{
    
    private BasicEngine colEngine;
    
    private BasicEngine cbEngine;
    
    
    /** Creates a new instance of ICHybridRecommender */
    public ICHybridRecommender() {

    }

    private void getEngines () throws Exception {
        colEngine = this.getEngineInstance(this.getConfigParameter(this.getClass().getName(),"COLLABORATIVE_ENGINE_CLASS"));
        colEngine.constructor(this.dbName);
        cbEngine = this.getEngineInstance(this.getConfigParameter(this.getClass().getName(),"CONTENTBASED_ENGINE_CLASS"));
        cbEngine.constructor(this.dbName);
    }
    
    public void makeModel () {
        try {
            this.getEngines();
            colEngine.makeModel();
            cbEngine.makeModel();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
    
    public List getRecommendationsByUser (User user)  {
        try {
            this.getEngines();

            //Collaborative Recommendations
            ArrayList colRecList = (ArrayList) colEngine.getRecommendationsByUser(user);

            //Content-Based Recomendations Recommendations
            ArrayList cbRecList = (ArrayList) cbEngine.getRecommendationsByUser(user);

            //Hybrid
            List hybridList = this.weightedCombination(colRecList,Double.parseDouble(this.getConfigParameter(this.getClass().getName(),"COLLABORATIVE_WEIGHT")),cbRecList,Double.parseDouble(this.getConfigParameter(this.getClass().getName(),"CONTENTBASED_WEIGHT")));        
            return hybridList;
        
         } catch (Exception e) {
            e.printStackTrace();
            return  null;
        }
    }
    
    public List getRecommendationsByItem (Item item) {
        try {
            this.getEngines();
        
            //Collaborative Recommendations
            ArrayList colRecList = (ArrayList) colEngine.getRecommendationsByItem(item);

            //Content-Based Recomendations Recommendations
            ArrayList cbRecList = (ArrayList) cbEngine.getRecommendationsByItem(item);

            //Hybrid
            List hybridList = this.weightedCombination(colRecList,Double.parseDouble(this.getConfigParameter(this.getClass().getName(),"COLLABORATIVE_WEIGHT")),cbRecList,Double.parseDouble(this.getConfigParameter(this.getClass().getName(),"CONTENTBASED_WEIGHT")));        
            return hybridList;
        } catch (Exception e) {
            e.printStackTrace();
            return  null;
        }

    }
    
    private List weightedCombination (ArrayList colList, double colWeight , ArrayList cbList, double cbWeight) throws Exception {        
        
        ArrayList hList = new ArrayList();
        int sizeHList = 0;
        int colCount=0; int cbCount=0;
        double currentColWeight=0; double currentCBWeight=0;
        while ((sizeHList<Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_SIZE_HYBRID_LIST")))&&((colCount<colList.size())||(cbCount<cbList.size()))) {
            //currentColWeight=(colList.size()-colCount)*colWeight;
            //currentCBWeight=(cbList.size()-cbCount)*cbWeight;    
            currentColWeight=(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_SIZE_COLLABORATIVE_LIST"))-colCount)*colWeight;
            currentCBWeight=(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_SIZE_CONTENTBASED_LIST"))-cbCount)*cbWeight;    
            //Collaborative
            if ((colCount<colList.size())&&((cbCount>=cbList.size())||(currentColWeight>=currentCBWeight))) {
                hList.add(colList.get(colCount));
                colCount++;                
            }
            //Content-Based
            if ((cbCount<cbList.size())&&((colCount>=colList.size())||(currentCBWeight>currentColWeight))) {
                hList.add(cbList.get(cbCount));
                cbCount++;                
            }            
        }
        return hList;
    }
    
}
