/*
 * ICLuceneRecommender.java
 *
 * Created on 6 de Fevereiro de 2007, 09:00
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.unicamp.ic.recommender.lucene;

import br.unicamp.ic.recommender.engine.BasicEngine;
import br.unicamp.ic.recommender.dao.ItemDAO;
import br.unicamp.ic.recommender.entity.Item;
import br.unicamp.ic.recommender.config.DBConfig;
import br.unicamp.ic.recommender.entity.User;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.FilterIndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.similar.MoreLikeThis;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
/**
 *
 * @author daniel
 */
public class ICLuceneRecommender extends BasicEngine {
    
    /**
     * Creates a new instance of ICLuceneRecommender
     */
    public ICLuceneRecommender() {
    }
    
    public File getIndexDir() {
        String pathIndexDir = this.getConfigParameter(this.getClass().getName(),"INDEXES_LOCAL");
        File indexDir = new File (pathIndexDir);
        return indexDir;
    }
    
    private String notNullField (String strField) {
        if (strField==null) {
            return new String("");
        } else {
            return strField;                    
        }
    }
    
    public void makeModel() {      
        try {
              IndexWriter writer = new IndexWriter(this.getIndexDir(),new StandardAnalyzer(), true);
              writer.setUseCompoundFile(false);

              ItemDAO itemDAO = new ItemDAO(this.dbName);
              Object[] items = itemDAO.selectAll();

              for (int i=0; i<items.length; i++) {
                  Item item = (Item) items[i];
                  Document doc = new Document();
                  doc.add(new Field ("id",Integer.toString(item.getItemId()),Field.Store.YES,Field.Index.NO));
                  doc.add(new Field ("title",item.getTitle(),Field.Store.YES,Field.Index.UN_TOKENIZED));

                  doc.add(new Field ("description",this.notNullField(item.getDescription()),Field.Store.NO,Field.Index.TOKENIZED));
                  doc.add(new Field ("textcontents",this.notNullField(item.getTextcontents()),Field.Store.NO,Field.Index.TOKENIZED));
                  doc.add(new Field ("creator",this.notNullField(item.getCreator()),Field.Store.YES,Field.Index.UN_TOKENIZED));

                  writer.addDocument(doc);
              }

              System.out.println("Indexao Completa. Total de itens indexados: " + writer.docCount());
              writer.optimize();
              writer.close();
          } catch (IOException eIO) {
            System.out.println("Erro no acesso aos ndices!");
            eIO.printStackTrace();
        }
  }
      
       
 public List getRecommendationsByItem(Item item)  {    
       try {
              Directory fsDir = FSDirectory.getDirectory(this.getIndexDir(),false);
              IndexSearcher indexSearcher = new IndexSearcher(fsDir);

              MoreLikeThis moreLikeThis = new MoreLikeThis(indexSearcher.getIndexReader());
              moreLikeThis.setFieldNames( new String[] {"title","creator","description","textcontents"} );

              moreLikeThis.setMinDocFreq(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MIN_DOCUMENT_FREQ_CONTENT_BY_ITEM")));
              moreLikeThis.setMinWordLen(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MIN_WORD_LENGTH_BY_ITEM")));

              ItemDAO itemDAO = new ItemDAO(this.dbName);
              item = (Item) itemDAO.selectByKey(new Integer(item.getItemId()));

              String strQuery = new String (this.notNullField(item.getTitle()) + " " + this.notNullField(item.getCreator()) + " " + this.notNullField(item.getDescription()) + " " + this.notNullField(item.getTextcontents()));	  
              Query query = moreLikeThis.like(new StringReader(strQuery));          
              Hits hits = indexSearcher.search(query);

              List recList = new ArrayList();
              int sizeList = 0;

              for (int i=0; ((i<hits.length())&&(sizeList<=Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_SIZE_CONTENTBASED_LIST")))); i++) {                
                    Document doc = hits.doc(i);
                    if (Integer.parseInt(doc.get("id"))==item.getItemId()) {
                        continue;
                    }                
                    Item currentItem = (Item) itemDAO.selectByKey(new Integer(doc.get("id")));               
                    recList.add(currentItem);            
                    sizeList++;
              }          
              return recList;
          } catch (IOException eIO) {
            System.out.println("Erro no acesso aos ndices!");
            eIO.printStackTrace();
            return null;
        }
 }       
      

 
    public List getRecommendationsByUser(User user) { 
        try {
              Directory fsDir = FSDirectory.getDirectory(this.getIndexDir(),false);
              IndexSearcher indexSearcher = new IndexSearcher(fsDir);

              MoreLikeThis moreLikeThis = new MoreLikeThis(indexSearcher.getIndexReader());
              moreLikeThis.setFieldNames( new String[] {"title","creator","description","textcontents"} );
              moreLikeThis.setMinDocFreq(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MIN_DOCUMENT_FREQ_CONTENT_BY_ITEM")));
              moreLikeThis.setMinWordLen(Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MIN_WORD_LENGTH_BY_ITEM")));

              ItemDAO itemDAO = new ItemDAO(this.dbName);
              Object[] ratedItems = itemDAO.getRatedItems(user);
              if (ratedItems==null) {
                  return new ArrayList();
              }
              StringBuffer strQuery = new StringBuffer();

              for (int i=0; ((i<ratedItems.length)&&(i<=Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_ITEMS_RATED_BY_USER")))); i++) {                
                     Item item = (Item) ratedItems[i];
                     strQuery.append (this.notNullField(item.getTitle()) + " " + this.notNullField(item.getCreator()) + " " + this.notNullField(item.getDescription()) + " " + this.notNullField(item.getTextcontents()));	  
              }    

              Query query = moreLikeThis.like(new StringReader(strQuery.toString()));          
              Hits hits = indexSearcher.search(query);

              List recList = new ArrayList();
              int sizeList=0;         
              boolean isRated;
              for (int i=0; ((i<hits.length())&&(sizeList<=Integer.parseInt(this.getConfigParameter(this.getClass().getName(),"MAX_SIZE_CONTENTBASED_LIST")))); i++) {                
                    Document doc = hits.doc(i);
                    isRated = false;               
                    for (int j=0; (j<ratedItems.length); j++) {                
                         Item item = (Item) ratedItems[j];
                         if (Integer.parseInt(doc.get("id"))==item.getItemId()) {
                            isRated=true; 
                            break;
                         }                
                    }
                    if (!isRated) {
                        Item currentItem = (Item) itemDAO.selectByKey(new Integer(doc.get("id")));               
                        recList.add(currentItem);                                
                        sizeList++;                     
                    }                                            
              }          
              return recList;
        } catch (IOException eIO) {
            System.out.println("Erro no acesso aos ndices!");
            eIO.printStackTrace();
            return null;
        }
 }       
    
}
